/*{***************************************************************************
**
** +COMPONENT_ID+  SIMBAD_CLIENT
** +LANGUAGE+      ANSI_C
**
** Filename:       simbad_client2.h
** 
** Purpose :       This component provides an API interfacing to the SIMBAD
**                 nameserver by connection over the internet. The 2 functions
**                 provided by the interface are designed for the search
**                 by position and search by name (alias) respectively.
**
** Authors :       <.....>
**
**}**************************************************************************/

/* Start of condition to prevent multiple inclusion of this header file. */
#ifndef _SIMBAD_CLIENT_H_
#define _SIMBAD_CLIENT_H_	0

#ifndef _ARGS
#ifdef __STDC__
#define _ARGS(A)        A 	/* ANSI */
#else
#define _ARGS(A)        ()  	/* Traditional */
#endif
#endif


/*{---------------------------------------------------------------------------
--
-- +FUNCTION+   simbad_pos
--
-- Purpose:     For a named astronomical object, get the position
--              defined by right ascension and declination coordinates.
--              The position is always indicated with respect to the same
--              equinox: J2000 (TBD).
--
-- Returns:     The precision of coordinates from -1 (no position),
--              0 (degree) ... 4 (arcsec) ... 7 (mas)
--
-- Remarks:     This functions establishes a connection to SIMBAD server.
--
--}--------------------------------------------------------------------------*/
int simbad_pos (
    char *p_obj_name,	/* IN: Name of astronomical object. */ 
    double pos[2]	/* OUT: Position from SIMBAD        */
);

/*{---------------------------------------------------------------------------
--
-- +FUNCTION+   simbad_ids
--
-- Purpose:     For a named astronomical object, get a list of
--              SIMBAD aliases for all catalogues.
--
-- Return:      A string with all names separated by a comma
--
-- Remarks:     This functions establishes a connection to SIMBAD server.
--
--}--------------------------------------------------------------------------*/
char *simbad_ids (
    char *p_obj_name	/* IN: Name of astronomical object */
);

int simbad_exit();
/* End of condition to prevent multiple inclusion of this header file. */
#endif	
